body[data-md-color-scheme="default"] .puml.light {
    display: block;
}
body[data-md-color-scheme="slate"] .puml.light {
    display: none;
}

body[data-md-color-scheme="default"] .puml.dark {
    display: none;
}
body[data-md-color-scheme="slate"] .puml.dark {
    display: block;
}

.puml {
    display: block;
    position: relative;
}

.puml .diagram {
    width: 100% !important;
    min-height: 300px !important;
}

.puml .diagram.wide-svg {
    height: auto !important;
}
